@@ -166,9 +166,21 @@ class ConnectionManager : public folly::DelayedDestruction,
       double pct,
       const std::function<bool(ManagedConnection*)>& filter);
 
-  size_t getNumConnections() const {
-    return conns_.size();
-  }
+  /**
+   * Returns total number of connections managed by this ConnectionManager.
+   * This includes active + idle connections.
+   */
+  size_t getNumConnections() const;
+
+  /**
+   * Returns number of active connections in the ConnectionManger.
+   */
+  size_t getNumActiveConnections() const;
+
+  /**
+   * Returns number of idle connection in the ConnectionManger.
+   */
+  size_t getNumIdleConnections() const;
 
   template <typename F>
   void forEachConnection(F func) {
@@ -197,6 +209,19 @@ class ConnectionManager : public folly::DelayedDestruction,
    */
   size_t dropIdleConnections(size_t num);
 
+  /**
+   * Drop conections based on idle timeout.
+   *
+   * @param targetIdleTimeMS The target idle timeout for all connections.
+   * @param droppedConnectionsCB Callback will be called at the end of the
+   *    method with number of dropped connections as input.
+   *
+   * @return Number of connections dropped.
+   */
+  size_t dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds targetIdleTimeMs,
+      const std::function<void(size_t)>& droppedConnectionsCB = [](size_t) {});
+
   /**
    * reportActivity is meant to be called when significant activity occurred on
    * the connection. reportActivity puts the connection in the front of the
@@ -212,7 +237,14 @@ class ConnectionManager : public folly::DelayedDestruction,
   void onDeactivated(ManagedConnection& conn) override;
 
  protected:
-  ~ConnectionManager() override = default;
+  ~ConnectionManager() override {
+    // These timeouts are expected to be canceled in the event base thread, so
+    // we attempt to enforce this.
+    if (drainHelper_.isScheduled()) {
+      eventBase_->runImmediatelyOrRunInEventBaseThreadAndWait(
+          [this]() { drainHelper_.cancelTimeout(); });
+    }
+  }
 
  private:
   enum class ShutdownState : uint8_t {
@@ -349,5 +381,7 @@ class ConnectionManager : public folly::DelayedDestruction,
    * time is less than idleConnEarlyDropThreshold_.
    */
   std::chrono::milliseconds idleConnEarlyDropThreshold_;
+
+  size_t idleConnections_{0};
 };
 } // namespace wangle
