@@ -130,10 +130,14 @@ void MockConnection::closeWhenIdleImpl() {
 TEST_F(ConnectionManagerTest, testShutdownSequence) {
   InSequence enforceOrder;
 
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
   // activate one connection, it should not be exempt from notifyPendingShutdown
   cm_->onActivated(*conns_.front());
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
   // make sure the idleIterator points to !end
   cm_->onDeactivated(*conns_.back());
+  EXPECT_EQ(cm_->getNumIdleConnections(), 1);
+  EXPECT_EQ(cm_->getNumActiveConnections(), 64);
   for (const auto& conn : conns_) {
     EXPECT_CALL(*conn, notifyPendingShutdown());
   }
@@ -152,6 +156,7 @@ TEST_F(ConnectionManagerTest, testRemoveDrainIterator) {
 
   // activate one connection, it should not be exempt from notifyPendingShutdown
   cm_->onActivated(*conns_.front());
+  EXPECT_EQ(cm_->getNumActiveConnections(), 66);
   for (size_t i = 0; i < conns_.size() - 1; i++) {
     EXPECT_CALL(*conns_[i], notifyPendingShutdown());
   }
@@ -160,10 +165,10 @@ TEST_F(ConnectionManagerTest, testRemoveDrainIterator) {
   eventBase_.runInLoop([&] {
     // deactivate the drain iterator
     cm_->onDeactivated(*conn65);
+    EXPECT_EQ(cm_->getNumIdleConnections(), 1);
     // remove the drain iterator
     cm_->removeConnection(conn66);
-    // deactivate the new drain iterator, now it's the end of the list
-    cm_->onDeactivated(*conn65);
+    EXPECT_EQ(cm_->getNumIdleConnections(), 1);
   });
   cm_->initiateGracefulShutdown(std::chrono::milliseconds(50));
   // Schedule a loop callback to remove the connection pointed to by the drain
@@ -201,12 +206,16 @@ TEST_F(ConnectionManagerTest, testIdleGraceTimeout) {
 TEST_F(ConnectionManagerTest, testDropAll) {
   InSequence enforceOrder;
 
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
   for (const auto& conn : conns_) {
     EXPECT_CALL(*conn, dropConnection(_))
         .WillOnce(Invoke(
             [&](const std::string&) { cm_->removeConnection(conn.get()); }));
   }
+  EXPECT_EQ(cm_->getNumIdleConnections(), 0);
   cm_->dropAllConnections();
+  EXPECT_EQ(cm_->getNumActiveConnections(), 0);
+  EXPECT_EQ(cm_->getNumIdleConnections(), 0);
 }
 
 TEST_F(ConnectionManagerTest, testDropEstablishedFilterDropAll) {
@@ -218,8 +227,10 @@ TEST_F(ConnectionManagerTest, testDropEstablishedFilterDropAll) {
 
   // We want to drop 100% of the connections and filter returns true all the
   // time as a result we should drop all the requests
-  cm_->dropEstablishedConnections(
-      1, [](ManagedConnection* /*mangedConn*/) { return true; });
+  cm_->dropEstablishedConnections(1, [](ManagedConnection* managedConn) {
+    (void)managedConn->getPeerAddress();
+    return true;
+  });
 }
 
 TEST_F(ConnectionManagerTest, testDropEstablishedVerifyOrder) {
@@ -234,8 +245,10 @@ TEST_F(ConnectionManagerTest, testDropEstablishedVerifyOrder) {
 
   // We want to drop 100% of the connections and filter returns true all the
   // time as a result we should drop all the requests
-  cm_->dropEstablishedConnections(
-      1, [](ManagedConnection* /*mangedConn*/) { return true; });
+  cm_->dropEstablishedConnections(1, [](ManagedConnection* managedConn) {
+    (void)managedConn->getPeerAddress();
+    return true;
+  });
 
   // Initially connection will be added in decreasing order highest to lowest,
   // it will be N, N - 1, N - 2, ..., 1, 0, thats because we loop from (0, N)
@@ -249,17 +262,22 @@ TEST_F(ConnectionManagerTest, testDropEstablishedVerifyOrder) {
 }
 
 TEST_F(ConnectionManagerTest, testDropFilterDropNone) {
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
   for (const auto& conn : conns_) {
     EXPECT_CALL(*conn, dropConnection(_)).Times(0);
   }
 
   // We want to drop 100% of the connections but filter returns false all the
   // time as a result we should see zero dropped connection
-  cm_->dropEstablishedConnections(
-      1, [](ManagedConnection* /*mangedConn*/) { return false; });
+  cm_->dropEstablishedConnections(1, [](ManagedConnection* managedConn) {
+    (void)managedConn->getPeerAddress();
+    return false;
+  });
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
 }
 
 TEST_F(ConnectionManagerTest, testDropFilterDropHalfOnly) {
+  EXPECT_EQ(cm_->getNumActiveConnections(), 65);
   std::vector<int> identifiers{1, 4, 10, 30};
   auto containsIdentifier = [&identifiers](int num) {
     return std::find(identifiers.begin(), identifiers.end(), num) !=
@@ -275,6 +293,7 @@ TEST_F(ConnectionManagerTest, testDropFilterDropHalfOnly) {
   }
 
   auto filter = [&containsIdentifier](ManagedConnection* managedConn) -> bool {
+    (void)managedConn->getPeerAddress();
     MockConnection* mockConn = dynamic_cast<MockConnection*>(managedConn);
     if (containsIdentifier(mockConn->identifier_)) {
       return true;
@@ -283,6 +302,7 @@ TEST_F(ConnectionManagerTest, testDropFilterDropHalfOnly) {
   };
 
   cm_->dropEstablishedConnections(1, filter);
+  EXPECT_EQ(cm_->getNumActiveConnections(), 61);
 }
 
 // We call onDeactivated on connection with identifier 30, as a result this
@@ -307,7 +327,10 @@ TEST_F(ConnectionManagerTest, testDropFilterTraverseTillIdleIterator) {
     }
   }
 
-  auto filter = [](ManagedConnection* managedConn) -> bool { return true; };
+  auto filter = [](ManagedConnection* managedConn) -> bool {
+    (void)managedConn->getPeerAddress();
+    return true;
+  };
   cm_->dropEstablishedConnections(1, filter);
 }
 
@@ -322,6 +345,7 @@ TEST_F(ConnectionManagerTest, testDropPercent) {
     removeConn(conns_.begin()->get());
   }
   EXPECT_EQ(100, cm_->getNumConnections());
+  EXPECT_EQ(100, cm_->getNumActiveConnections());
 
   // Drop 20% of connections.
   double pct = 0.2;
@@ -337,6 +361,7 @@ TEST_F(ConnectionManagerTest, testDropPercent) {
   // Make sure they are gone.
   EXPECT_EQ(0, numToDrop);
   EXPECT_EQ(80, cm_->getNumConnections());
+  EXPECT_EQ(cm_->getNumActiveConnections(), 80);
 
   // Then drop 50% of the remaining 80 connections.
   pct = 0.5;
@@ -351,6 +376,7 @@ TEST_F(ConnectionManagerTest, testDropPercent) {
   // Make sure those are gone as well.
   EXPECT_EQ(0, numToDrop);
   EXPECT_EQ(40, cm_->getNumConnections());
+  EXPECT_EQ(40, cm_->getNumActiveConnections());
 }
 
 TEST_F(ConnectionManagerTest, testDrainPercent) {
@@ -424,6 +450,101 @@ TEST_F(ConnectionManagerTest, testDropIdle) {
         .WillRepeatedly(Return(std::chrono::milliseconds(100)));
   }
 
+  int idleCount_{0};
+  // Mark the first half of the connections idle
+  for (size_t i = 0; i < conns_.size() / 2; i++) {
+    EXPECT_EQ(cm_->getNumIdleConnections(), idleCount_);
+    cm_->onDeactivated(*conns_[i]);
+    idleCount_++;
+  }
+  EXPECT_EQ(
+      idleCount_, cm_->getNumConnections() - cm_->getNumActiveConnections());
+  // reactivate conn 0
+  cm_->onActivated(*conns_[0]);
+  EXPECT_EQ(cm_->getNumIdleConnections(), idleCount_ - 1);
+  // remove the first idle conn
+  cm_->removeConnection(conns_[1].get());
+  // Given we reactivated idle connection we expect number of idle connections
+  // to be one less than before
+  EXPECT_EQ(cm_->getNumIdleConnections(), idleCount_ - 2);
+
+  InSequence enforceOrder;
+
+  // Expect the remaining idle conns to drop
+  for (size_t i = 2; i < conns_.size() / 2; i++) {
+    EXPECT_CALL(*conns_[i], dropConnection(_))
+        .WillOnce(Invoke([this, i](const std::string&) {
+          cm_->removeConnection(conns_[i].get());
+        }));
+  }
+
+  cm_->dropIdleConnections(conns_.size());
+}
+
+TEST_F(ConnectionManagerTest, testDropIdleConnectionsBasedOnIdleTimeDropAll) {
+  // Make every connection to be idle for 100 milliseconds
+  for (const auto& conn : conns_) {
+    EXPECT_CALL(*conn, getIdleTime())
+        .WillRepeatedly(Return(std::chrono::milliseconds(100)));
+  }
+
+  // Mark every connection idle
+  for (size_t i = 0; i < conns_.size(); i++) {
+    cm_->onDeactivated(*conns_[i]);
+  }
+
+  InSequence enforceOrder;
+
+  // Expect every connection to be dropped
+  for (size_t i = 0; i < conns_.size(); i++) {
+    EXPECT_CALL(*conns_[i], dropConnection(_))
+        .WillOnce(Invoke([this, i](const std::string&) {
+          cm_->removeConnection(conns_[i].get());
+        }));
+  }
+
+  bool callbackCalled{false};
+  cm_->dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds(99),
+      [this, &callbackCalled](size_t numConnectionsDropped) {
+        EXPECT_EQ(numConnectionsDropped, conns_.size());
+        callbackCalled = true;
+      });
+  EXPECT_TRUE(callbackCalled);
+}
+
+TEST_F(ConnectionManagerTest, testDropIdleConnectionsBasedOnIdleTimeDropNone) {
+  // Make every connection to be able for 100 milliseconds
+  for (const auto& conn : conns_) {
+    EXPECT_CALL(*conn, getIdleTime())
+        .WillRepeatedly(Return(std::chrono::milliseconds(100)));
+  }
+
+  // Mark every connection idle
+  for (size_t i = 0; i < conns_.size(); i++) {
+    cm_->onDeactivated(*conns_[i]);
+  }
+
+  InSequence enforceOrder;
+
+  // Expect no connection to be dropped
+  for (size_t i = 0; i < conns_.size(); i++) {
+    EXPECT_CALL(*conns_[i], dropConnection(_)).Times(0);
+  }
+
+  cm_->dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds(100), [](size_t numConnectionsDropped) {
+        EXPECT_EQ(numConnectionsDropped, 0);
+      });
+}
+
+TEST_F(ConnectionManagerTest, testDropIdleConnectionsBasedOnIdleTimeDropHalf) {
+  for (const auto& conn : conns_) {
+    // Set everyone to be idle for 100ms
+    EXPECT_CALL(*conn, getIdleTime())
+        .WillRepeatedly(Return(std::chrono::milliseconds(100)));
+  }
+
   // Mark the first half of the connections idle
   for (size_t i = 0; i < conns_.size() / 2; i++) {
     cm_->onDeactivated(*conns_[i]);
@@ -443,7 +564,52 @@ TEST_F(ConnectionManagerTest, testDropIdle) {
         }));
   }
 
-  cm_->dropIdleConnections(conns_.size());
+  bool callbackCalled{false};
+  cm_->dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds(99),
+      [this, &callbackCalled](size_t numConnectionsDropped) {
+        EXPECT_EQ(numConnectionsDropped, conns_.size() / 2 - 2);
+        callbackCalled = true;
+      });
+  EXPECT_TRUE(callbackCalled);
+}
+
+TEST_F(ConnectionManagerTest, testDropIdleConnectionsBasedOnIdleTimeEarlyStop) {
+  size_t halfConnectionsSize = conns_.size() / 2;
+  for (size_t i = halfConnectionsSize; i < conns_.size(); i++) {
+    EXPECT_CALL(*conns_[i], getIdleTime())
+        .WillRepeatedly(Return(std::chrono::milliseconds(10)));
+  }
+
+  for (size_t i = 0; i < halfConnectionsSize; i++) {
+    // Set everyone to be idle for 100ms
+    EXPECT_CALL(*conns_[i], getIdleTime())
+        .WillRepeatedly(Return(std::chrono::milliseconds(100)));
+  }
+
+  // Mark every connection idle
+  for (size_t i = 0; i < conns_.size(); i++) {
+    cm_->onDeactivated(*conns_[i]);
+  }
+
+  InSequence enforceOrder;
+
+  // Expect the remaining idle conns to drop
+  for (size_t i = 0; i < halfConnectionsSize; i++) {
+    EXPECT_CALL(*conns_[i], dropConnection(_))
+        .WillOnce(Invoke([this, i](const std::string&) {
+          cm_->removeConnection(conns_[i].get());
+        }));
+  }
+
+  bool callbackCalled{false};
+  cm_->dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds(99),
+      [halfConnectionsSize, &callbackCalled](size_t numConnectionsDropped) {
+        EXPECT_EQ(numConnectionsDropped, halfConnectionsSize);
+        callbackCalled = true;
+      });
+  EXPECT_TRUE(callbackCalled);
 }
 
 TEST_F(ConnectionManagerTest, testAddDuringShutdown) {
